// blues-main.cc -- the main function and other necessary functions for a 
//      radio drum program are in this file.  Include this file in a file 
//	that contains the user-defined funections.  Look at the file 
//	/user/c/craig/lang/c/drum/template.cc as an example.
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Mon Apr  8 11:57:03 PDT 1996
// Last Modified: Sun Apr 21 14:37:34 PDT 1996
// Filename:      /user/c/craig/lang/c/drum/main.cc
// Syntax:        C++ (Gnu C++ compiler: /Local/Unix/bin/g++)
//
////////////////////////////////////////////////////////////////////////////////
// 
// Variables for the radio baton user functions:
//
//////////////////////////
                        //
int transposition = 0;  //
int zlevel = 70;        //
                        //
////////////////////////////////////////////////////////////////////////////////

int tempIn;

#include <iostream.h>
#include <ctype.h>
#include <stdio.h>
#include "drum-setup.h"
#include "classDefinitions.h"
#include "options.h"

// global objects:
Baton drum;                     // the object to interact with the radio drum
MidiIO midi;                    // the object to interact with the MIDI IO
MidiBuffer mb; 		        // a hacking struct for dealing with drum & midi

long t_time = 0;                // where midi time is stored in milliseconds

// global variables for options:
int  gargc;                     // global variable for argc
char **gargv;                   // gloval variable for argv
int  argp = 1;                  // pointer to current argument to be processed
int port = PORT_A;              // [-a | -b]: port to use for MIDI
int quiet = 0;                  // boolean test for quiet (for FixedPolicy)
                                //    (not yet implemented)

// function declarations for this file:
void exitUsage(void);           // prints usage statement and exits
void getOptions(void);          // processes options from the command line
void checkMidiInput(void);      // look for drum command in MIDI input
void commandmode(void);         // command mode for typing stuff on keyboard
void printcommands(void);       // print the commands available in commandmode
void silence(void);             // send note off to all 16x128 midi notes
void fastsilence(int);          // send note off to notes in specified channel

// user defined functions:
void description(void);         // short description of what program does
void programhelp(void);         // explaination of radio drum variables
void initialization(void);      // action to take at beginning of program
void mainloopalgorithms(void);	// main loop actions of user
void stick1trig(void);          // action after stick1 triggers
void stick2trig(void);          // action after stick2 triggers
void b14plustrig(void);         // action after b14+ button is pushed
void b15plustrig(void);         // action after b15+ button is pushed
void stick1pollresponse(void);  // action after stick1 returns a poll
void stick2pollresponse(void);  // action after stick2 returns a poll
void potspollresponse(void);    // action after pots return a poll
void cleanup(void);		// action to take before exiting program


////////////////////////////////////////////////////////////////////////////////

int main(int argc, char* argv[]) {
   gargc = argc;
   gargv = argv;
   getOptions();
   if (argp != argc) exitUsage();

   midi.open(port);

   initialization();
   commandmode();
   while (1) {
      t_time = midi_time();
      checkMidiInput();
      mainloopalgorithms();
   }
}

////////////////////////////////////////////////////////////////////////////////


//////////////////////////////
//
// checkMidiInput -- look at the midi input to see if a command
//	has been sent by the radio drum.
//

void checkMidiInput(void) {
   midi.readBuffer(mb);
   if (mb.message) {
      mb.message = 0;
      switch (mb.data[1]) {
         case 101:                        // poll from stick1
            drum.storestick1p(); 
            stick1pollresponse(); 
            break;
         case 102:                        // poll from stick2
            drum.storestick2p(); 
            stick2pollresponse(); 
            break;
         case 103:                        // poll from pots
            drum.storepots(); 
            potspollresponse(); 
            break;
         case 111:                        // trigger from stick1
            drum.storestick1t(); 
            stick1trig(); 
            break;
         case 112:                        // trigger from stick2
            drum.storestick2t(); 
            stick2trig(); 
            break;
         case 114:                        // trigger from b14+
            b14plustrig();
            break;
         case 115:                        // trigger from b15+
            b15plustrig();
            break;
      }
   }
}



//////////////////////////////
//
// commandmode
//

#define MAX_COMMAND_STRING_LENGTH 256

void commandmode(void) {
   char keybuf[MAX_COMMAND_STRING_LENGTH];
   int keych;
   int length;

   while (1) {
      printcommands();
      scanf("%S", keybuf);
      length = strlen(keybuf);
      if (length >= MAX_COMMAND_STRING_LENGTH) {
         cerr << "Error: command line input was too long " << endl;
         exit(1);
      }
      keych = keybuf[length-1];    // read last char in input string
      switch (tolower(keych)) {
         case 's': silence(); break;
         case 'f': fastsilence(CH_1); break;
         case 'd': description(); break;
         case 'j': midi.sysEx(0x2d); break;
         case 'l':                 //set stick levels at current stick positions
            drum.setStickLevels();
            cerr << "Hit levels set at current stick positions \n";
            break;
         case 'q':
            cleanup();
            exit(0);
            break;
         case 'p':
            cerr << "Start of PLAY mode. Press B14+ to return to COMMAND mode.\n";
            // Note: user now gets to redefine the B14+ button trigger.
            return;
            break;
         case 'h': programhelp(); break;
         case 't': 
            tempIn = 1000;
            cout << "Enter semitones from middle c (between -12 and 12): ";
            while (tempIn < -12 || tempIn > 12) {
               cin >> tempIn;
               if (tempIn < -12 || tempIn > 12) {
                  cout << "Invalid transposition!  Enter new transposition: ";
               }
            }
            transposition = tempIn;
            cout << "Transposition is: " << transposition << endl;
            break;
         case 'i': 
            tempIn = 1000;
            cout << "Enter z level for imporv off point (between 0 and 127): ";
            while (tempIn < 0 || tempIn > 127) {
               cin >> tempIn;
               if (tempIn < 0 || tempIn > 127) {
                  cout << "Invalid level!  Enter new level: ";
               }
            }
            zlevel = tempIn;
            cout << "Level is: " << zlevel << endl;
            break;
      } 
   } 
} 



//////////////////////////////
//
// silence, fastsilence
//

void silence(void) {
   for (int i=0; i<2; i++) for (int j=0; j<128; j++) midi.play(i,j,0);
}

void fastsilence(int channel) {
   silence();
}



//////////////////////////////
//
// printcommands
//

void printcommands(void) {
   cout 
   << "###############################################################\n"
   << "# Commands:                                                   #\n"
   << "#    j) Jaffe-Schloss mode set d) description of program      #\n"
   << "#    q) quit                   l) set stick levels            #\n"
   << "#    s) silence                f) fast silence                #\n"
   << "#    p) play                   h) help                        #\n"
   << "#    t) transposition          i) improv off level            #\n"
   << "###############################################################\n"
   << "                                                               \n"
   << "type a command and press <Return>: "
   ;
}



//////////////////////////////
//
// getOptions
//

void getOptions(void) {
   int position = 1;
   while (argp < gargc && gargv[argp][0] == '-') {
      while (1) {
         switch (gargv[argp][position]) {
            case 'a':
               port = PORT_A;
               NO_ARGUMENTS;
               break;
            case 'b':
               port = PORT_B;
               NO_ARGUMENTS;
               break;
            case 'q':
               quiet = 1;
               NO_ARGUMENTS;
               break;
            case '\0':
               argp++;
               break;
            default:
               cerr << "Unknown option: " << gargv[argp] << endl;
               exitUsage();
         }
         break;
      }
      position = 1;
      argp++;
   }
}



//////////////////////////////
//
// exitUsage
//

void exitUsage(void) {
   cerr << "\nUsage: " << gargv[0] << " [-a | -b]\n"
        << "   Options are:\n"
        << "      -a = use port A for MIDI.\n"
        << "      -b = use port B for MIDI.\n"
        << endl;
   exit(1);
}


////////////////////////////////////////////////////////////////////////////////

/* User functions to be inserted at this point.  See the file
 *	/user/c/craig/lang/c/drum/template.cc for example definitions of 
 * 	these functions: 
 *
 *    description:            short description of what program does
 *    programhelp:            explaination of radio drum variables
 *    initialization:         action to take at beginning of program
 *    mainloopalgorithms:     main loop actions of user
 *    stick1trig:             action after stick1 triggers
 *    stick2trig:             action after stick2 triggers
 *    b14plustrig:            action after b14+ button is pushed
 *    b15plustrig:            action after b15+ button is pushed
 *    stick1pollresponse:     action after stick1 returns a poll
 *    stick2pollresponse:     action after stick2 returns a poll
 *    potspollresponse:       action after pots return a poll
 */

