// guitar.cc -- Simulate a virtual guitar on the radio drum.  Idea
//	taken from a fellow students final project in Music 120, Winter 1996.
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Tue Apr  9 21:27:50 PDT 1996
// Last Modified: Thu Apr 18 13:09:49 PDT 1996
// Filename:      /user/c/craig/lang/c/drum/examples/guitar.cc
// Syntax:        C++ (Gnu C++ compiler: /Local/Unix/bin/g++)
// $Smake:        g++ -O -o %b %f -I../include -L../lib %
//                -ldrum -lmidi -loptions && strip %b
//                Type "../bin/smake FILENAME" to compile this file.
//

#include <math.h>				 // for function abs
#include "main.cc"
#include "psr-520.h"   				 // Timbre Names for PSR-520
#include "guitar-fingerings.h"


enum strings {string1 = 3, string2 = 5, string3 = 7, string4 = 9, 
              string5 = 11, string6 = 13};


/***************** Beginning of user function *****************************/

// There is a different Midi Channel for every string:
Note  st1(CH_1, PSR_ACOUSTIC_GUITAR_NYLON);        // Bass   E string
Note  st2(CH_2, PSR_ACOUSTIC_GUITAR_NYLON);        //        A string
Note  st3(CH_3, PSR_ACOUSTIC_GUITAR_NYLON);        //        D string
Note  st4(CH_4, PSR_ACOUSTIC_GUITAR_NYLON);        //        G string
Note  st5(CH_5, PSR_ACOUSTIC_GUITAR_NYLON);        //        B string
Note  st6(CH_6, PSR_ACOUSTIC_GUITAR_NYLON);        // Treble E string

int xpos1 = 0;		// position of baton 1 in the y-axis
int ypos2 = 0;		// position of baton 2 in the y-axis
int new1Q = 0;		// boolean for if baton 1 in new box
int new2Q = 0;		// boolean for if baton 2 in new box
int test = 0;

void setStrings(int n1, int n2, int n3, int n4, int n5, int n6) {
   if (n1 != X) st1.setKeyno(E2 + n1);
   else {        
      st1.off();
      st1.setKeyno(-1);
   }
   if (n2 != X) st2.setKeyno(A2 + n2);
   else {        
      st2.off();
      st1.setKeyno(-1);
   }
   if (n3 != X) st3.setKeyno(D3 + n3);
   else {        
      st3.off();
      st1.setKeyno(-1);
   }
   if (n4 != X) st4.setKeyno(G3 + n4);
   else {        
      st4.off();
      st1.setKeyno(-1);
   }
   if (n5 != X) st5.setKeyno(B3 + n5);
   else {        
      st5.off();
      st1.setKeyno(-1);
   }
   if (n6 != X) st6.setKeyno(E4 + n6);
   else {        
      st6.off();
      st1.setKeyno(-1);
   }
}

/*---------------- Documentation functions -------------------------------*/

void description(void) {
   cout
   << "______________________________________________________________ \n"
   << "| Description:                                                |\n"
   << "|                                                             |\n"
   << "|                                                             |\n"
   << "|                                                             |\n"
   << "|                                                             |\n"
   << "|                          -- Craig Stuart Sapp, 10 Apr 1996  |\n"
   << "---------------------------------------------------------------\n"
   ;
}


void programhelp(void) {
   cout
   << ">                                                              \n"
   << "> Baton 1:                                                     \n"
   << ">   trig: Start the hot/cold game                              \n"
   << ">    x1p: X-axis note -- higher if warmer, lower if colder     \n"
   << ">    y1p: Y-axis note -- higher if warmer, lower if colder     \n"
   << ">    z1p: Z-axis note -- higher if warmer, lower if colder     \n"
   << "> Baton 2:                                                     \n"
   << ">   trig: Stop the hot/cold game                               \n"
   << ">                                                              \n"
   << ">    p1p: Attack velocity of x direction voice                 \n"
   << ">    p2p: Attack velocity of y direction voice                 \n"
   << ">    p3p: Attack velocity of z direction voice                 \n"
   << ">   +b14: Command mode                                         \n"
   << ">   +b15: This help screen                                     \n"
   << ">                                                              \n"
   ;
}

/*---------------- Initialization algorithms -----------------------------*/

void initialization(void) { 
   description();
   drum.setPollRate(10);       // poll rate of radio drum set to 10 milliseconds
   setStrings(OPEN_STRINGS);
   st1.sendTimbre();
   st2.sendTimbre();
   st3.sendTimbre();
   st4.sendTimbre();
   st5.sendTimbre();
   st6.sendTimbre();
}


/*---------------- Main loop algorithms ----------------------------------*/

void mainloopalgorithms(void) { 
   drum.checkPollBatons();

   // if baton 1 moves onto a new string:
   if (drum.z1p > 70 && abs(new1Q) >= 2 && (xpos1 % 2) == 1) {
      switch (xpos1) {
         case string1:  st1.setVelocity(drum.z1p); st1.play(); break;
         case string2:  st2.setVelocity(drum.z1p); st2.play(); break;
         case string3:  st3.setVelocity(drum.z1p); st3.play(); break;
         case string4:  st4.setVelocity(drum.z1p); st4.play(); break;
         case string5:  st5.setVelocity(drum.z1p); st5.play(); break;
         case string6:  st6.setVelocity(drum.z1p); st6.play(); break;
      }
      new1Q = OFF;
   }
}
/*---------------- Trigger response functions ----------------------------*/

void stick1trig(void) { }

void stick2trig(void) {
   switch (drum.x2ts(0, 9)) {
      case 0: setStrings(C_); break;
      case 1: setStrings(G_); break;
      case 2: setStrings(F_); break;
      case 3: setStrings(G7_); break;
      case 4: setStrings(Am_); break;
      case 5: setStrings(Bm7b5_D_); break;
      case 6: setStrings(Dm_); break;
      case 7: setStrings(Fma7_); break;
      case 8: setStrings(Cma7_); break;
      case 9: setStrings(Bdim_D_); break;
   }
}


void b14plustrig(void) { 
   setStrings(DAMPED_STRINGS);
   commandmode(); 
}

void b15plustrig(void) { programhelp(); }

/*---------------- Poll response algorithms ------------------------------*/

void stick1pollresponse(void) { 
   xpos1 = drum.x1ps(0,15); 
   test = drum.cx1p(0,15);
   new1Q += drum.cx1p(0,15);
   if (drum.z1p < 50) {
      new1Q = 0;
   }
}

void stick2pollresponse(void) { 
}

void potspollresponse(void) { }

/*---------------- Actions to take just before leaving program -----------*/

void cleanup(void) { }

/***************** End of user functions **********************************/

